# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg

from euca2ools.commands.cloudformation import CloudFormationRequest


class DescribeStacks(CloudFormationRequest):
    DESCRIPTION = 'Describe one or more stacks'
    ARGS = [Arg('StackName', metavar='STACK', nargs="?",
                help='limit results to a single stack'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the stacks' info")]
    LIST_TAGS = ['Outputs', 'Parameters', 'Stacks']

    def print_result(self, result):
        for stack in result['Stacks']:
            self.print_stack(stack)
            if self.args['show_long']:
                print stack.get('StackId')
                print stack.get('NotificationARNs')
            if stack.get('Parameters'):
                for param in stack.get('Parameters'):
                    self.print_parameter(param)
            if stack.get('Outputs'):
                for output in stack.get('Outputs'):
                    self.print_output(output)
